// Generated by CoffeeScript 1.10.0
(function() {
  var CanvasPhash, Promise, bitCount, crypto, fs, getImageFromPath, getPixels, getRegionPixels, newBuffer, read;

  Promise = require('bluebird');

  getPixels = require('get-pixels');

  getPixels = Promise.promisify(getPixels);

  fs = require('fs');

  read = Promise.promisify(fs.readFile);

  crypto = require('crypto');

  newBuffer = function(size, encoding) {
    var e, error;
    try {
      return new Buffer(size, encoding);
    } catch (error) {
      e = error;
      return Buffer.alloc(size, encoding);
    }
  };

  bitCount = function(i) {
    i = i - ((i >>> 1) & 0x55555555);
    i = (i & 0x33333333) + ((i >>> 2) & 0x33333333);
    return (((i + (i >>> 4)) & 0x0F0F0F0F) * 0x01010101) >>> 24;
  };

  getImageFromPath = function(path) {
    return getPixels(path).then(function(pixels) {
      pixels.width = pixels.shape[0];
      pixels.height = pixels.shape[1];
      pixels.channels = pixels.shape[2];
      return pixels;
    });
  };

  getRegionPixels = function(img, x, y, width, height) {
    var i, j, k, l, pixelNdx, ref, ref1, ref2, ref3, result;
    result = [];
    for (j = k = ref = y, ref1 = y + height - 1; ref <= ref1 ? k <= ref1 : k >= ref1; j = ref <= ref1 ? ++k : --k) {
      for (i = l = ref2 = x, ref3 = x + width - 1; ref2 <= ref3 ? l <= ref3 : l >= ref3; i = ref2 <= ref3 ? ++l : --l) {
        pixelNdx = j * img.width + i;
        result.push(img.data[4 * pixelNdx]);
        result.push(img.data[4 * pixelNdx + 1]);
        result.push(img.data[4 * pixelNdx + 2]);
        result.push(img.data[4 * pixelNdx + 3]);
      }
    }
    return result;
  };

  CanvasPhash = {
    readImage: getImageFromPath,
    getSHA256: function(path) {
      var pixelPromise;
      if (typeof path === 'string') {
        pixelPromise = getImageFromPath(path);
      } else {
        pixelPromise = Promise.resolve(path);
      }
      return pixelPromise.then(function(pixels) {
        return Promise.resolve(new Buffer(pixels.data));
      }).then(function(pixelBuffer) {
        var hash, sha256;
        sha256 = crypto.createHash("sha256");
        sha256.update(pixelBuffer);
        hash = sha256.digest("base64");
        return hash;
      });
    },
    getImageHash: function(path) {
      var pixelPromise;
      if (typeof path === 'string') {
        pixelPromise = getImageFromPath(path);
      } else {
        pixelPromise = Promise.resolve(path);
      }
      return pixelPromise.then(function(img) {
        var Mean, Median, a, array, avg, b, blockCol, blockHeight, blockNdx, blockRow, blockWidth, buffer, char, color, g, getMedian, hexString, imageData, k, l, m, o, pixelNdx, r, val;
        Mean = {
          r: [],
          g: [],
          b: [],
          a: 0
        };
        for (blockRow = k = 0; k <= 15; blockRow = ++k) {
          for (blockCol = l = 0; l <= 15; blockCol = ++l) {
            blockWidth = Math.floor(img.width / 16);
            blockHeight = Math.floor(img.height / 16);
            imageData = getRegionPixels(img, blockWidth * blockCol, blockHeight * blockRow, blockWidth, blockHeight);
            avg = {
              r: 0,
              g: 0,
              b: 0
            };
            for (pixelNdx = m = 0; m <= 255; pixelNdx = ++m) {
              a = imageData[4 * pixelNdx + 3];
              if (a === 0) {
                r = 0xff;
                b = 0xff;
                g = 0xff;
              } else {
                r = imageData[4 * pixelNdx + 0];
                b = imageData[4 * pixelNdx + 1];
                g = imageData[4 * pixelNdx + 2];
              }
              avg.r += (r - avg.r) / (pixelNdx + 1);
              avg.g += (g - avg.g) / (pixelNdx + 1);
              avg.b += (b - avg.b) / (pixelNdx + 1);
            }
            for (color in avg) {
              val = avg[color];
              Mean[color].push(val);
            }
          }
        }
        Median = {
          r: 0,
          g: 0,
          b: 0
        };
        getMedian = function(values) {
          values.sort(function(a, b) {
            return a - b;
          });
          return values[Math.floor(values.length / 2)];
        };
        for (color in Median) {
          array = Median[color];
          Median[color] = getMedian(Mean[color].slice(0));
        }
        buffer = newBuffer(128, 'utf8');
        hexString = "";
        for (blockNdx = o = 0; o <= 255; blockNdx = ++o) {
          char = 0;
          r = Mean.r[blockNdx];
          g = Mean.g[blockNdx];
          b = Mean.b[blockNdx];
          if (r >= Median.r) {
            char |= 0x4;
          }
          if (g >= Median.g) {
            char |= 0x2;
          }
          if (b >= Median.b) {
            char |= 0x1;
          }
          hexString += char.toString(16);
        }
        buffer.write(hexString, 0, 128, 'hex');
        return buffer;
      });
    },
    getHammingDistance: function(buffer1, buffer2) {
      var hammingDistance, k, n, x, xor, y;
      Buffer(xor = newBuffer(128, 'utf8'));
      hammingDistance = 0;
      for (n = k = 0; k <= 127; n = k += 4) {
        x = buffer1.readUInt32BE(n);
        y = buffer2.readUInt32BE(n);
        hammingDistance += bitCount(x ^ y);
      }
      return hammingDistance;
    }
  };

  module.exports = CanvasPhash;

}).call(this);
